import numpy

import Orange.data
from Orange.widgets.widget import OWWidget, Input, Output
from Orange.widgets.utils.widgetpreview import WidgetPreview
from Orange.widgets import gui, settings

class Constant(OWWidget):
    name = "Constant"
    description = "Returns a Matrix of the same dimension as the input filled with a constant value"
    icon = "icons/Constant.svg"
    priority = 20

    class Inputs:
        ReferenceMatrix = Input("Reference Matrix", numpy.ndarray)

    class Outputs:
        OutputMatrix = Output("Constant Matrix", numpy.ndarray)

    want_main_area = False
    Constant = settings.Setting(128)
    # [end-snippet-1]

    def __init__(self):
        super().__init__()
        # GUI
        # [start-snippet-3]
        box = gui.widgetBox(self.controlArea, "Info")
        self.infoReferenceMatrix = gui.widgetLabel(box, "No data on input yet, waiting to get something.")
        gui.lineEdit(self.controlArea, self, "Constant", box="Constant value to fill the output matrix",
                     callback=self.handleNewSignals,
                     valueType=type(numpy.uint8(128)))

        gui.separator(self.controlArea)

    @Inputs.ReferenceMatrix
    def set_OutputMatrix(self, ReferenceMatrix):
        if ReferenceMatrix is not None:
            self.infoReferenceMatrix.setText("input ReferenceMatrix is " + str(ReferenceMatrix.dtype))
            self.ReferenceMatrix = ReferenceMatrix
        else:
            self.infoReferenceMatrix.setText("No data on input yet, waiting to get something.")



    # def subtractor_type(self,block1, block2, type):
    #     b1 = Casting.caster(block1, type)
    #     b2 = Casting.caster(block2, type)
    #     res = numpy.subtract(b1, b2)
    #     return res, type
        
    def handleNewSignals(self):
        if ((self.Inputs.ReferenceMatrix is not None)):
            self.Outputs.OutputMatrix.send(numpy.full(self.ReferenceMatrix.shape, self.Constant, dtype="uint8"))

if __name__ == "__main__":
    WidgetPreview(Constant).run(numpy.uint8(128))


