import math

import numpy
import cv2 as cv
import Orange.data
from Orange.widgets.widget import OWWidget, Input, Output
from Orange.widgets.utils.widgetpreview import WidgetPreview
from Orange.widgets import gui, settings
weights = numpy.array(
         [[8, 16, 19, 22, 26, 27, 29, 34], [16, 16, 22, 24, 27, 29, 34, 37], [19, 22, 26, 27, 29, 34, 34, 38],[22, 22, 26, 27, 29, 34, 37, 40], [22, 26, 27, 29, 32, 35, 40, 48], [26, 27, 29, 32, 35, 40, 48, 58],[26, 27, 29, 34, 38, 46, 56, 69], [27, 29, 35, 38, 46, 56, 69, 83]])

class Reconstructor(OWWidget):
    name = "Reconstructor"
    description = "Reconstruct a set of values."
    icon = "icons/reconstructor.svg"
    priority = 20

    class Inputs:
        a = Input("A", numpy.ndarray)

    class Outputs:
        dqA = Output("DQ(A)", numpy.ndarray)

    want_main_area = False
    ptype = settings.Setting(0)
    Sq = settings.Setting(2)
    SqVals = range(2, 64, 2)
    MODES = ['Inter', 'Intra']

    def __init__(self):
        super().__init__()

        self.dataset = None
        self.sample = None

        box = gui.widgetBox(self.controlArea, "Info")
        self.infoa = gui.widgetLabel(box, "No data on input yet, waiting to get something.")
        gui.separator(self.controlArea)

        gui.comboBox(self, self, 'ptype', items=self.MODES, callback=self.handleNewSignals)

        gui.comboBox(self, self, 'Sq', items=self.SqVals, callback=self.handleNewSignals)

    @Inputs.a
    def set_data_a(self, a):
        if a is not None:
            self.infoa.setText("input A is correct")
            self.a = a

    # def dequant(self, image, predtype):
    #     weights = numpy.ones((8, 8))
    #     block = numpy.zeros((8, 8))
    #     blockq = numpy.zeros((8, 8))
    #     imares = image
    #     Sq = 16
    #     if predtype == 'Intra':
    #         weights = 16 * weights
    #     else:
    #         weights = numpy.array(
    #             [[8, 16, 19, 22, 26, 27, 29, 34], [16, 16, 22, 24, 27, 29, 34, 37], [19, 22, 26, 27, 29, 34, 34, 38],
    #              [22, 22, 26, 27, 29, 34, 37, 40], [22, 26, 27, 29, 32, 35, 40, 48], [26, 27, 29, 32, 35, 40, 48, 58],
    #              [26, 27, 29, 34, 38, 46, 56, 69], [27, 29, 35, 38, 46, 56, 69, 83]])
    #     dim = image.shape
    #     filas = dim[0]
    #     columnas = dim[1]
    #     for ypos in range(0, filas, 8):
    #         f_ini = ypos
    #         f_fin = min(ypos + 8, filas)
    #         for xpos in range(0, columnas, 8):
    #             c_ini = xpos
    #             c_fin = min(xpos + 8, columnas)
    #             # Ya tenemos los bloques
    #             block = image[f_ini:f_fin, c_ini:c_fin]
    #             blockq = (8 * block) / (weights * Sq)
    #             if predtype:
    #                 blockq[0, 0] = block[0, 0] * 8
    #                 blockq = numpy.floor(blockq)
    #             else:
    #                 blockq[0, 0] = block[0, 0] / 8
    #                 blockq = numpy.around(blockq)
    #
    #             imares[f_ini:f_fin, c_ini:c_fin] = blockq
    #     return cv.cvtColor(imares, cv.COLOR_GRAY2BGR)
    def dequanth261(self):
        th = 8
        q = self.q
        img = self.a.astype("double")
        block = numpy.zeros((8, 8))
        blockdq = numpy.zeros((8, 8))
        imares = img.copy()
        dim = img.shape
        filas = dim[0]
        columnas = dim[1]
        for ypos in range(0, filas, 8):
            f_ini = ypos
            f_fin = min(ypos + 8, filas)
            for xpos in range(0, columnas, 8):
                c_ini = xpos
                c_fin = min(xpos + 8, columnas)
                block = img[f_ini:f_fin, c_ini:c_fin]
                if (self.MODES[self.ptype] == "Intra"):
                    blockdqt = block * weights*q/8
                    blockdqt[0,0] = block[0,0]*8
                    blockdqt = numpy.round(blockdqt)
                    blockdqtaux = blockdqt.copy()
                    blockdqtaux[blockdqt%2 == 0]= blockdqt[blockdqt%2 == 0] - numpy.sign(blockdqt[blockdqt%2 == 0])
                    blockdqt = blockdqtaux
                if (self.MODES[self.ptype] == "Inter"):
                    blockdqt = ((2*block+numpy.sign(block))*16*q)/16
                    blockdqt = numpy.round(blockdqt)
                    blockdqtaux = blockdqt.copy()
                    blockdqtaux[blockdqt%2 == 0]= blockdqt[blockdqt%2 == 0] - numpy.sign(blockdqt[blockdqt%2 == 0])
                    blockdqt = blockdqtaux
                # if (self.MODES[self.ptype] == "Intra"):
                #     blockdq = numpy.where(block!=0, blockdq, block*q - ((numpy.sign(block)*q)/2))
                # if (self.MODES[self.ptype] == "Inter"):
                #     blockdq = block*q
                   
                imares[f_ini:f_fin, c_ini:c_fin] = blockdqt
        return imares

    def handleNewSignals(self):
        if (self.a is not None):
            self.q = self.SqVals[self.Sq]
            newdq = self.dequanth261()
            self.Outputs.dqA.send(newdq.astype("double"))


if __name__ == "__main__":
    WidgetPreview(VideoFile).run()