import numpy as np
import cv2 as cv
import Orange.data
from Orange.widgets.widget import OWWidget, Input, Output
from Orange.widgets.utils.widgetpreview import WidgetPreview
from Orange.widgets import gui, settings

class Entropy(OWWidget):
    name = "Entropy"
    description = "Shows the entropy of a given image"
    icon = "icons/Entropy.svg"
    priority = 20

    class Inputs:
        a = Input("Image (estimated)", np.ndarray)

    want_main_area = False
    def __init__(self):
        super().__init__()

        self.dataset = None
        self.sample = None
        box = gui.widgetBox(self.controlArea, "Stats:")
        self.infoE = gui.widgetLabel(box, "Entropy = ")

    @Inputs.a
    def set_data_a(self, a):
        if a is not None:
            self.a = a

    def caster(self, inimag, type):
        res = inimag.astype(type)
        return res

    def operate(self):
        base = None
        imaa = self.a.copy()
        if (imaa.ndim > 2):
            imaa = imaa[:, :, 0]
        # img = self.caster(self.a, 'uint8')
        imaa = imaa.flatten()
        n_labels = len(imaa)

        if n_labels <= 1:
            return 0

        value, counts = np.unique(imaa, return_counts=True)
        probs = counts / n_labels
        n_classes = np.count_nonzero(probs)

        if n_classes <= 1:
            return 0

        ent = 0.

        # Compute entropy
        base = 2 if base is None else base
        for i in probs:
            ent -= i * np.log2(i)

        self.infoE.setText("Entropy = " + str(np.round(ent,4)) + " bpp")

    def handleNewSignals(self):
        if self.Inputs.a is not None:
            self.operate()


if __name__ == "__main__":
    array = np.linspace(0, 1, 256 * 256)

    # reshape to 2d
    mat = np.reshape(array, (256, 256))

    # Creates PIL image
    img = np.uint8(mat * 255)
    WidgetPreview(Entropy).run(img)