import numpy as np
import math
import cv2 as cv
import time
import Orange.data
from Orange.widgets.widget import OWWidget, Input, Output
from Orange.widgets.utils.widgetpreview import WidgetPreview
from Orange.widgets import gui, settings
from AnyQt import QtWidgets
class Est_Comp(OWWidget):
    name = "Est&Comp"
    description = "Estimates the motion of a sequence given two images: "
    icon = "icons/Est_Comp.svg"
    priority = 20

    class Inputs:
        a = Input("Current image", np.ndarray)
        b = Input("Reference image", np.ndarray)

    class Outputs:
        imaOut = Output("Predicted image", np.ndarray)
        imaDif = Output("Error image (Current - Predicted)", np.ndarray)
        vecOut = Output("Motion vectors", np.ndarray)

    want_main_area = False
    modeS = settings.Setting(0)
    modeC = settings.Setting(0)
    p = settings.Setting(4)
    q = settings.Setting(4)
    M = settings.Setting(8)
    N = settings.Setting(8)
    L = settings.Setting(0)
    Subpixel = settings.Setting(0)
    Pyramidal = settings.Setting(0)
    hierx = settings.Setting(4)
    hiery = settings.Setting(4)
    SEARCH_TYPE = ['Exhaustive', 'Logarithmic', 'Logarithmic Zero End', 'Hierarchical']
    COMP_TYPE = ['MAE', 'MSE', 'NCFF']
    def __init__(self):
        super().__init__()

        self.box = gui.widgetBox(self.controlArea, "Info")
        from AnyQt.QtGui import QIntValidator
        gui.lineEdit(self.controlArea, self, "p", box="Search area width (p)",
                     callback=self.handleNewSettings,
                     valueType=int, validator=QIntValidator())

        gui.lineEdit(self.controlArea, self, "q", box="Search area height (q)",
                     callback=self.handleNewSettings,
                     valueType=int, validator=QIntValidator())

        gui.lineEdit(self.controlArea, self, "M", box="Block width (M)",
                     callback=self.handleNewSettings,
                     valueType=int, validator=QIntValidator())

        gui.lineEdit(self.controlArea, self, "N", box=" Block Height (N)",
                     callback=self.handleNewSettings,
                     valueType=int, validator=QIntValidator())

        gui.comboBox(self, self, 'modeS', items=self.SEARCH_TYPE,callback=self.handleNewSettings)
        self.hierxField = gui.lineEdit(self.controlArea, self, "hierx", box="dx",
                     callback=self.handleNewSettings,
                     valueType=int, validator=QIntValidator())
        self.hieryField = gui.lineEdit(self.controlArea, self, "hiery",  box="dy",
                     callback=self.handleNewSettings,
                     valueType=int, validator=QIntValidator())

        gui.comboBox(self, self, 'modeC', items=self.COMP_TYPE,callback=self.handleNewSettings)
        gui.checkBox(
            self.controlArea, self, "Subpixel", "1/2 pixel precision", callback=self.handleNewSettings
        )
        gui.checkBox(
            self.controlArea, self, "Pyramidal", "Pyramidal", callback=self.handleNewSettings
        )
        self.pyramidalLevelField = gui.lineEdit(self.controlArea, self, "L", box="Levels of pyramidal algorithm (L)",
                     callback=self.handleNewSettings,
                     valueType=int, validator=QIntValidator())
        self.infoexectime = gui.widgetLabel(self.box, "Execution time = ")

        self.ApplyButton = gui.button(self, self, "Apply",
                                                callback=self.handleNewSignals)
        
        self.UpdateUI()
        

    @Inputs.a
    def set_data_a(self, a):
        if a is not None:
            self.a = a

    @Inputs.b
    def set_data_b(self, b):
        if b is not None:
            self.b = b

    def search(self):
        current = self.a.astype("double")
        reference = self.b.astype("double")
        if(current.ndim > 2):
            if (np.max(np.abs(current[:,:,0] - current[:,:,1])) != 0):
                msg = QtWidgets.QMessageBox()
                msg.setIcon(QtWidgets.QMessageBox.Critical)
                msg.setText("Error")
                msg.setInformativeText('The current frame could not be loaded, please, ensure that it has only one channel')
                msg.setWindowTitle("Error")
                msg.exec_()
            else:
                current = current[:,:,0]
                
        if(reference.ndim > 2):
            if (np.max(np.abs(reference[:,:,0] - reference[:,:,1])) != 0):
                msg = QtWidgets.QMessageBox()
                msg.setIcon(QtWidgets.QMessageBox.Critical)
                msg.setText("Error")
                msg.setInformativeText('The reference frame could not be loaded, please, ensure that it has only one channel')
                msg.setWindowTitle("Error")
                msg.exec_()
            else:
                reference = reference[:,:,0]
                
        currentorig = current
        p = self.p
        q = self.q
        M = self.M
        N = self.N
        L = self.L
        self.precision = 1
        hierar = [self.hierx, self.hiery]
        executiontime = time.time()
        referenceorig = reference.copy()
        if(self.Subpixel == True):
            self.precision = 0.5
            # p=p*2
            # q=q*2
            # M=M*2
            # N=N*2
            # hierar[0]=hierar[0]*2
            # hierar[1]=hierar[1]*2
            # referenceorig = reference.copy()
            # currentorig = current.copy()
            # width = int(currentorig.shape[1] * 2)
            # height = int(currentorig.shape[0] * 2)
            # newdim = (width, height)
            # current = cv.resize(current.copy(),newdim,interpolation = cv.INTER_LINEAR)
            # reference = cv.resize(reference.copy(),newdim,interpolation = cv.INTER_LINEAR)
        if(self.Pyramidal == True):
            Niter = self.L+1
            resultIma = np.zeros(referenceorig.shape, dtype="uint8")
            for iter in range(Niter):
                if iter == 0:
                    pL = math.ceil(p / (2**(L-iter)))
                    qL = math.ceil(q / (2**(L-iter)))
                    ML = math.ceil(M / (2**(L-iter)))
                    NL = math.ceil(N / (2**(L-iter)))
                    hierar0L = math.ceil(hierar[0] / (2**(L-iter)))
                    hierar1L = math.ceil(hierar[1] / (2**(L-iter)))
                    currentL = cv.resize(current.copy(),None,fx =1/(2**(L-iter)),fy=1/(2**(L-iter)))
                    referenceL = cv.resize(reference.copy(),None,fx =1/(2**(L-iter)),fy=1/(2**(L-iter)))

                    if (self.SEARCH_TYPE[self.modeS] == "Exhaustive"):
                        resultImaL, errorImaL, vectors = self.search_ex(currentL, referenceL, pL, qL, ML, NL, 1)
                    elif (self.SEARCH_TYPE[self.modeS] == "Logarithmic"):
                        resultImaL, errorImaL, vectors = self.search_log1(currentL, referenceL, pL, qL, ML, NL, 1)
                    elif (self.SEARCH_TYPE[self.modeS] == "Logarithmic Zero End"):
                        resultImaL, errorImaL, vectors = self.search_log2(currentL, referenceL, pL, qL, ML, NL, 1)
                    elif (self.SEARCH_TYPE[self.modeS] == "Hierarchical"):
                        resultImaL, errorImaL, vectors = self.search_hier(currentL, referenceL, [hierar0L,hierar1L], pL, qL, ML, NL, 1)
                    vectors = (vectors * (2**(L-iter))).astype(int)

                    for PosX, PosY, DimX, DimY in vectors.astype(int):
                        resultIma[PosX:PosX + N, PosY:PosY + M] = referenceorig[PosX + DimX:PosX + N + DimX,PosY + DimY:PosY + M + DimY]
                else:
                    pL = 1
                    qL = 1
                    ML = math.ceil(M / (2**(L-iter)))
                    NL = math.ceil(N / (2**(L-iter)))
                    currentL = cv.resize(current.copy(),None,fx =1/(2**(L-iter)),fy=1/(2**(L-iter)))
                    referenceL = cv.resize(resultIma.copy(),None,fx =1/(2**(L-iter)),fy=1/(2**(L-iter)))
                    resultImaL, errorImaL, vectorsL = self.search_ex(currentL, referenceL, pL, qL, ML, NL, 1)
                    #resultIma = ?
                    vectors[:,2:4] = vectors[:,2:4] + ((vectorsL[:,2:4])*(2**(L-iter))).astype(int)
                    #vectors = (vectors * (2**(L-iter))).astype(int)
                    for PosX, PosY, DimX, DimY in vectors.astype(int):
                        resultIma[PosX:PosX + N, PosY:PosY + M] = referenceorig[PosX + DimX:PosX + N + DimX,PosY + DimY:PosY + M + DimY]
                        
            errorIma = np.abs(currentorig - resultIma)
            errorIma = errorIma.astype("uint8")

                
                
        else:
            if (self.SEARCH_TYPE[self.modeS] == "Exhaustive"):
                resultIma, errorIma, vectors = self.search_ex(current, reference, p, q, M, N, self.precision)
            elif (self.SEARCH_TYPE[self.modeS] == "Logarithmic"):
                resultIma, errorIma, vectors = self.search_log1(current, reference, p, q, M, N, 1)
            elif (self.SEARCH_TYPE[self.modeS] == "Logarithmic Zero End"):
                resultIma, errorIma, vectors = self.search_log2(current, reference, p, q, M, N, 1)
            elif (self.SEARCH_TYPE[self.modeS] == "Hierarchical"):
                resultIma, errorIma, vectors = self.search_hier(current, reference, hierar, p, q, M, N, 1)
        # else:
        #     print('Search type not valid')
        
        # if(self.Subpixel == True):
        #     resultIma = np.zeros(referenceorig.shape, dtype="uint8")
        #     vectors = np.round(vectors/2)
        #     vectorPos = vectors[:,0:2]
        #     VectorDim =vectors[:,2:4]
        #     M = M*2
        #     N = N*2
        #     for PosX, PosY, DimX, DimY in vectors.astype(int):
        #         resultIma[PosX:PosX+N,PosY:PosY+M] = referenceorig[PosX+DimX:PosX+N+DimX,PosY+DimY:PosY+M+DimY]
        #         errorIma = np.abs(currentorig - resultIma)
        #         errorIma = errorIma.astype("uint8")

        executiontime = round(time.time()-executiontime,4)
        self.infoexectime.setText("Execution time = " + str(executiontime) + "s")
        return resultIma, errorIma, vectors
    def UpdateUI(self):
        if(self.SEARCH_TYPE[self.modeS] == "Hierarchical"):
            self.hieryField.setDisabled(False)
            self.hierxField.setDisabled(False)
        else:
            self.hieryField.setDisabled(True)
            self.hierxField.setDisabled(True)

        if(self.Pyramidal == True):
            self.pyramidalLevelField.setDisabled(False)

        else:
            self.pyramidalLevelField.setDisabled(True)

    def handleNewSettings(self):
        self.UpdateUI()
        self.ApplyButton.setDisabled(False)

    def handleNewSignals(self):
        self.UpdateUI()
        if ((self.Inputs.a is not None) and (self.Inputs.b is not None)):
            print("Searching...")
            newres, newerr, newvec = self.search()
            self.ApplyButton.setDisabled(True)
            self.Outputs.imaOut.send(newres)
            self.Outputs.imaDif.send(newerr)
            self.Outputs.vecOut.send(newvec)


    def cost(self, block1, block2):
        size = block1.shape
        dim1 = size[0]
        dim2 = size[1]
        res = 0
        NxM = dim1 * dim2
        if (self.COMP_TYPE[self.modeC] == "MAE"):
            res = np.abs(np.subtract(block1, block2))
            res = np.sum(res) / NxM
        if (self.COMP_TYPE[self.modeC] == "MSE"):
            res = np.power(np.subtract(block1, block2), 2)
            res = np.sum(res) / NxM
        if (self.COMP_TYPE[self.modeC] == "NCFF"):
            num = np.sum(block1 * block2)
            den1 = np.power(np.sum(np.power(block1, 2)), 0.5)
            den2 = np.power(np.sum(np.power(block2, 2)), 0.5)
            res = num / (den1 * den2)
        # else:
        #     print("Cost function not valid")
        return res


    def search_log1(self, current, reference, p, q, M, N,precision):
        # La mas restrictiva
        resultIma = current.copy()
        errorIma = reference
        vectors = np.array([])
        dim = current.shape
        filas = dim[0]
        columnas = dim[1]
        numberBlocksf = math.floor(filas / N)
        numberBlocksc = math.floor(columnas / M)
        for i in range(numberBlocksf):
            f0 = i * N
            fend = min(filas, i * N + N)
            refOrigin = [0, 0]
            for j in range(numberBlocksc):
                c0 = j * M
                cend = min(columnas, j * M + M)
                currentBlock = current[f0:fend, c0:cend]
                center = [f0, c0]
                bestCost = np.Inf
                bestBlock = np.zeros((N, M))
                kp = math.ceil(np.log2(p))
                kq = math.ceil(np.log2(q))
                dp = int(2 ** (kp - 1))
                dq = int(2 ** (kq - 1))
                # Podríamos usar el parámetro hierarchy para elegir que metodo utilizar
                while ((dp >= 1) and (dq >= 1)):  # No sería and?
                    #for disty in range(max(center[0] - dq, 0), min(dq + center[0], filas), dq):
                    #    for distx in range(max(center[1] - dp, 0), min(dp + center[1], columnas), dp):
                    #        refBlock = reference[disty:disty + N, distx:distx + M]
                    #        if (refBlock.shape != currentBlock.shape):
                    #            continue
                    
                    for iRef in range(center[0] - dq, center[0] + dq +1 ,dq):  # 2p+1
                        
                        if (iRef < 0 or ((iRef + N) > filas)):
                            # jump = jump + 1
                            continue
                        for cRef in range(center[1] - dp, center[1] + dp +1 ,dp):  # 2p+1
                            
                            if (cRef < 0 or ((cRef + M) > columnas)):
                                # jump = jump+1
                                continue
                            #print("CurrentBloq", f0,"x",c0," dq",dq," dp",dp,"iRef=",iRef,"  cRef=",cRef)
                            fRef0 = iRef
                            fRefend = fRef0 + N
                            cRef0 = cRef
                            cRefend = cRef0 + M
                            refBlock = reference[fRef0:fRefend, cRef0:cRefend]
                            comp = self.cost(currentBlock, refBlock)
                            if comp <= bestCost:
                                bestCost = comp
                                bestBlock = refBlock.copy()
                                refOrigin = [fRef0, cRef0]
                                #refOrigin = [disty, distx]
                            #if comp == bestCost:
                            #    if (np.sqrt(np.power((refOrigin[0] - f0), 2) + np.power((refOrigin[1] - c0), 2))) > (
                            #    np.sqrt(np.power((fRef0 - f0), 2) + np.power((cRef0 - c0), 2))):
                            #        bestBlock = refBlock.copy()
                            #        refOrigin = [fRef0, cRef0]
                    dp = int(math.floor(dp / 2))
                    dq = int(math.floor(dq / 2))
                    center = refOrigin

                resultIma[f0:fend, c0:cend] = bestBlock

                vector = np.array([f0, c0, refOrigin[0] - f0, refOrigin[1] - c0])
                if vectors.size == 0:
                    vectors = vector.copy()
                else:
                    vectors = np.vstack((vectors, vector))
        errorIma = np.abs(current - resultIma)
        rmse = np.sqrt(np.mean((current - resultIma) ** 2))
        return np.round(resultIma).astype("uint8"), errorIma.astype("uint8"), vectors

    def search_log2(self, current, reference, p, q, M, N, precision):
        resultIma = current.copy()
        errorIma = reference
        vectors = np.array([])
        dim = current.shape
        filas = dim[0]
        columnas = dim[1]
        numberBlocksf = math.floor(filas / N)
        numberBlocksc = math.floor(columnas / M)
        for i in range(numberBlocksf):
            f0 = i * N
            fend = min(filas, i * N + N)
            refOrigin = [0, 0]
            for j in range(numberBlocksc):
                c0 = j * M
                cend = min(columnas, j * M + M)
                currentBlock = current[f0:fend, c0:cend]
                center = [f0, c0]
                bestCost = np.Inf
                bestBlock = np.zeros((N, M))
                kp = math.ceil(np.log2(p))
                kq = math.ceil(np.log2(q))
                dp = int(2 ** (kp - 1))
                dq = int(2 ** (kq - 1))
                while ((dp > 0) and (dq > 0)):
                    dp = int(dp)
                    dq = int(dq)
                    for disty in range(max(center[0] - dq, 0), min(dq + center[0], filas), dq):
                        for distx in range(max(center[1] - dp, 0), min(dp + center[1], columnas), dp):
                            refBlock = reference[disty:disty + N, distx:distx + M]
                            if (refBlock.shape != currentBlock.shape):
                                continue
                            comp = self.cost(currentBlock, refBlock)
                            if comp < bestCost:
                                bestCost = comp
                                bestBlock = refBlock
                                refOrigin = [disty, distx]
                            if comp == bestCost:
                                if (pow((refOrigin[0] - f0) ** 2 + (refOrigin[1] - c0) ** 2, 1 / 2)) > (
                                        pow((disty - f0) ** 2 + (distx - c0) ** 2, 1 / 2)):
                                    bestBlock = refBlock
                                    refOrigin = [disty, distx]
                    dp = math.floor(dp / 2)
                    dq = math.floor(dq / 2)

                resultIma[f0:fend, c0:cend] = bestBlock

                vector = np.array([f0, c0, refOrigin[0] - f0, refOrigin[1] - c0])
                if vectors.size == 0:
                    vectors = vector.copy()
                else:
                    vectors = np.vstack((vectors, vector))
        errorIma = np.abs(current - resultIma)
        rmse = np.sqrt(np.mean((current - resultIma) ** 2))
        return np.round(resultIma).astype("uint8"), errorIma.astype("uint8"), vectors

    def search_ex(self, current, reference, p, q, M, N, precision):
        resultIma = current.copy()
        errorIma = reference
        vectors = np.array([])
        dim = current.shape
        filas = dim[0]
        columnas = dim[1]
        numberBlocksf = math.ceil(filas / N)
        numberBlocksc = math.ceil(columnas / M)
        if (precision != 1):
            refResized = cv.resize(reference, None, fx=2, fy=2, interpolation=cv.INTER_LINEAR)
        for i in range(numberBlocksf):
            f0 = i * N
            fend = min(filas, f0 + N)
            refOrigin = [0, 0]
            for j in range(numberBlocksc):
                c0 = j * M
                cend = min(columnas, c0 + M)
                currentBlock = current[f0:fend, c0:cend]
                center = [f0, c0]
                bestCost = np.Inf
                bestBlock = np.zeros((fend-f0, cend-c0))
                for iRef in range(center[0] - q, center[0] + q + 1):  # 2p+1
                    if (iRef < 0 or ((iRef + fend-f0) > filas)):
                        # jump = jump + 1
                        continue
                    for cRef in range(center[1] - p, center[1] + p + 1):  # 2p+1
                        if (cRef < 0 or ((cRef + cend-c0) > columnas)):
                            # jump = jump+1
                            continue
                        fRef0 = iRef
                        fRefend = fRef0 + fend-f0
                        cRef0 = cRef
                        cRefend = cRef0 + cend-c0

                        refBlock = reference[fRef0:fRefend, cRef0:cRefend]

                        comp = self.cost(currentBlock, refBlock)
                        if comp < bestCost:
                            bestCost = comp
                            bestBlock = refBlock
                            refOrigin = [fRef0, cRef0]
                        elif comp == bestCost:
                            if (np.sqrt(np.power((refOrigin[0] - f0), 2) + np.power((refOrigin[1] - c0), 2))) > (
                            np.sqrt(np.power((fRef0 - f0), 2) + np.power((cRef0 - c0), 2))):
                                bestBlock = refBlock
                                refOrigin = [fRef0, cRef0]


                if (precision != 1):
                    bestBlock, refOrigin = self.sub_search(currentBlock,bestBlock,refResized, refOrigin, M, N,bestCost)

                vector = np.array([f0, c0, refOrigin[0] - f0, refOrigin[1] - c0])
                if vectors.size == 0:
                    vectors = vector.copy()
                else:
                    vectors = np.vstack((vectors, vector))

                resultIma[f0:fend, c0:cend] = bestBlock

        errorIma = np.abs(current - resultIma)
        rmse = np.sqrt(np.mean((current - resultIma) ** 2))
        return np.round(resultIma).astype("uint8"), errorIma.astype("uint8"), vectors

    def sub_search(self,currentBlock,bestBlock, resizedRef, refOrigin, M, N,bestCost):
        dim = resizedRef.shape
        filas, columnas = resizedRef.shape
        NewrefOrigin =refOrigin
        for dimf in [-1,0,1]:
            for dimc in [-1,0,1]:
                if (dimf == 0 and dimc == 0): continue
                fRef0 = refOrigin[0]*2 + dimf
                fRefend = min(fRef0 + N*2, filas)
                cRef0 = refOrigin[1]*2 + dimc
                cRefend = min(cRef0 + M*2, columnas)
                refBlock = resizedRef[fRef0:fRefend, cRef0:cRefend]
                refBlock = refBlock[::2,::2]
                if(currentBlock.shape != refBlock.shape):
                    print("error in shape")
                    continue
                comp = self.cost(currentBlock, refBlock)
                if comp < bestCost:
                    print("Is better with subpixel! Old:", bestCost, " new:", comp)
                    bestBlock=refBlock
                    bestCost = comp
                    NewrefOrigin = [fRef0/2, cRef0/2]
        return bestBlock, NewrefOrigin

    def search_hier(self, current, reference, hierarchy, p, q, M, N, precision):
        resultIma = current.copy()
        errorIma = reference
        vectors = np.array([])
        dim = current.shape
        filas = dim[0]
        columnas = dim[1]
        numberBlocksf = math.ceil(filas / N)
        numberBlocksc = math.ceil(columnas / M)
        hierarx = hierarchy[0]
        hierary = hierarchy[1]
        for i in range(numberBlocksf):
            f0 = i * N
            fend = min(filas, f0 + N)
            refOrigin = [0, 0]
            for j in range(numberBlocksc):
                c0 = j * M
                cend = min(columnas, c0 + M)
                currentBlock = current[f0:fend, c0:cend]
                center = [f0, c0]
                p_min = c0 - p
                p_max = c0 + p 
                q_min = f0 - q
                q_max = f0 + q 
                bestCost = np.Inf
                N_2 = fend - f0
                M_2 = cend-c0
                bestBlock = np.zeros((N_2, M_2))
                X_1=range(center[0] - math.floor(q / hierary)*hierary,
                                   center[0] + math.floor(q / hierary)*hierary + 1, hierary)
                Y_1 = range(center[1] - math.floor(p / hierarx)*hierarx,
                                       center[1] + math.floor(p / hierarx)*hierarx + 1, hierarx)
                for iRef in range(center[0] - math.floor(q / hierary)*hierary,
                                   center[0] + math.floor(q / hierary)*hierary + 1, hierary):
                    if (iRef < 0 or (iRef + N_2) > filas):
                        continue
                    for cRef in range(center[1] - math.floor(p / hierarx)*hierarx,
                                       center[1] + math.floor(p / hierarx)*hierarx + 1, hierarx):
                        if (cRef < 0 or (cRef + M_2) > columnas):
                            continue
                        fRef0 = iRef
                        fRefend = fRef0 + N_2
                        cRef0 = cRef
                        cRefend = cRef0 + M_2

                        refBlock = reference[fRef0:fRefend, cRef0:cRefend]
                        comp = self.cost(currentBlock, refBlock)

                        if comp < bestCost:
                            bestCost = comp
                            bestBlock = refBlock.copy()
                            refOrigin = [fRef0, cRef0]
                        elif comp == bestCost:
                            if (np.sqrt(np.power((refOrigin[0] - f0), 2) + np.power((refOrigin[1] - c0), 2))) > (
                            np.sqrt(np.power((fRef0 - f0), 2) + np.power((cRef0 - c0), 2))):
                                bestBlock = refBlock
                                refOrigin = [fRef0, cRef0]
                center = [refOrigin[0], refOrigin[1]]
                for iRef in range(center[0] - (hierary-1), center[0] + hierary):  # 2p+1
                    if (iRef < 0 or ((iRef + N_2) > filas) or iRef < q_min or iRef > q_max):
                        # jump = jump + 1
                        continue
                    for cRef in range(center[1] - (hierarx-1), center[1] + hierarx):  # 2p+1
                        if (cRef < 0 or ((cRef + M_2) > columnas) or cRef < p_min or cRef > p_max):
                            # jump = jump+1
                            continue
                        fRef0 = iRef
                        fRefend = fRef0 + N_2
                        cRef0 = cRef
                        cRefend = cRef0 + M_2

                        refBlock = reference[fRef0:fRefend, cRef0:cRefend]

                        comp = self.cost(currentBlock, refBlock)
                        if comp < bestCost:
                            bestCost = comp
                            bestBlock = refBlock.copy()
                            refOrigin = [fRef0, cRef0]
                        elif comp == bestCost:
                            if (np.sqrt(np.power((refOrigin[0] - f0), 2) + np.power((refOrigin[1] - c0), 2))) > (
                            np.sqrt(np.power((fRef0 - f0), 2) + np.power((cRef0 - c0), 2))):
                                bestBlock = refBlock
                                refOrigin = [fRef0, cRef0]
                resultIma[f0:fend, c0:cend] = bestBlock

                vector = np.array([f0, c0, refOrigin[0] - f0, refOrigin[1] - c0])
                if vectors.size == 0:
                    vectors = vector.copy()
                else:
                    vectors = np.vstack((vectors, vector))
        errorIma = np.abs(current - resultIma)
        rmse = np.sqrt(np.mean((current - resultIma) ** 2))
        return np.round(resultIma).astype("uint8"), errorIma.astype("uint8"), vectors


if __name__ == "__main__":
    array = np.linspace(0, 1, 144 * 176*3)

    # reshape to 2d
    mat = np.reshape(array, (144, 176,3))

    # Creates PIL image
    img = np.uint8(mat * 150)
    WidgetPreview(Est_Comp).run(set_data_a=img,set_data_b=np.uint8(np.round(img/2)))
