import numpy
import cv2 as cv
import Orange.data
from orangewidget.widget import OWBaseWidget, Input, Output, gui
from orangewidget.utils.widgetpreview import WidgetPreview
from orangewidget import gui, settings
from AnyQt import QtCore
from orangewidget.settings import Setting
from AnyQt import QtWidgets, QtGui
class FrameSelector(OWBaseWidget):
    name = "FrameSelector"
    description = "Selects the frame of a given image."
    icon = "icons/Frame.svg"
    priority = 10
    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    #   with a fixed non resizable geometry.
    resizing_enabled = False
    
    class Outputs:
        frame = Output("Frame", numpy.ndarray)

    class Inputs:
        url = Input("Video Path", str)
    
    url = ""
    number = settings.Setting(1)
    proportion = settings.Setting(50)
    commitOnChange = settings.Setting(0)
    def __init__(self):
        super().__init__()
        box = gui.widgetBox(self.controlArea, "Information")
        from AnyQt.QtGui import QIntValidator
        gui.lineEdit(self.controlArea, self, "number", "Enter a number",
                     box="Number",
                     callback=self.number_changed,
                     valueType=int, validator=QIntValidator(),tooltip = "Enter the frame number in this box.")

        self.infoState = gui.widgetLabel(box, "Information:")
        #self.number_changed()
        
        
    @Inputs.url
    def set_frame(self, url):
        print(url)
        self.url = url
        self.number_changed()
    
    def number_changed(self):
        frame = numpy.zeros((255, 255,3), dtype='uint8')
        if (self.url != ""):
            cap = cv.VideoCapture(self.url)
            cap.set(cv.CAP_PROP_POS_FRAMES,self.number-1)
            ret, frame = cap.read()
            if ((not ret) or (self.number<1)):
                msg = QtWidgets.QMessageBox()
                msg.setIcon(QtWidgets.QMessageBox.Critical)
                msg.setText("Error")
                msg.setInformativeText('The frame could not be loaded, please, ensure that the frame number is in the range:[ 1 ,'+str((cap.get(cv.CAP_PROP_FRAME_COUNT))) + ' ]')
                msg.setWindowTitle("Error")
                msg.exec_()
                InfoText = "Error loading the image from:" + self.url
            else:
                InfoText = "Frame " + str(self.number) + " loaded successfully"
            if(frame.ndim > 2):
                if (numpy.max(numpy.abs(frame[:,:,0] - frame[:,:,1])) == 0):   
                    frame = frame[:,:,0]
        else:
            InfoText = "The input is empty"
        self.infoState.setText(InfoText)
        self.Outputs.frame.send(frame)