import numpy as np
import cv2 as cv
import Orange.data
from Orange.widgets.widget import OWWidget, Input, Output
from Orange.widgets.utils.widgetpreview import WidgetPreview
from Orange.widgets import gui, settings
import heapq


class Huffman(OWWidget):
    name = "Huffman"
    description = "Calculates the bpp of the Huffman code of a given data"
    icon = "icons/Huffman.svg"
    priority = 20
    img =None

    class Inputs:
        a = Input("Image (estimated)", np.ndarray)

    want_main_area = False
    def __init__(self):
        super().__init__()

        box = gui.widgetBox(self.controlArea, "Stats:")
        self.infoE = gui.widgetLabel(box, "bpp = ")
        self.the_codes = dict()

    @Inputs.a
    def set_data_a(self, a):
        if a is not None:
            self.img = a




    def calc_freq(self,data):
        freq_dict = {}
        data = data.flatten()
        for d in data:
            if d in freq_dict:
                freq_dict[d] += 1
            else:
                freq_dict[d] = 1
        return freq_dict

    def huffman_encode(self,freq_dict):
        heap = [[weight, [symbol, ""]] for symbol, weight in freq_dict.items()]
        heapq.heapify(heap)
        while len(heap) > 1:
            lo = heapq.heappop(heap)
            hi = heapq.heappop(heap)
            for pair in lo[1:]:
                pair[1] = '0' + pair[1]
            for pair in hi[1:]:
                pair[1] = '1' + pair[1]
            heapq.heappush(heap, [lo[0] + hi[0]] + lo[1:] + hi[1:])
        huff_dict = sorted(heapq.heappop(heap)[1:], key=lambda p: (len(p[-1]), p))
        return huff_dict

    def calc_total_bits(self,huff_dict, freq_dict):
        total_bits = 0
        for symbol, code in huff_dict:
            total_bits += len(code) * freq_dict[symbol]
        return total_bits

    def huffman_coding(self,data):
        freq_dict = self.calc_freq(data)
        huff_dict = self.huffman_encode(freq_dict)
        total_bits = self.calc_total_bits(huff_dict, freq_dict)
        return huff_dict, total_bits

    def handleNewSignals(self):
        if self.Inputs.a is not None:
            img = self.img
            s = img.shape
            if len(s) < 2:
                return (0, np.array([]), "Input error")

            if len(s) > 3:
                return (0, np.array([]), "Input error")

            if len(s) == 3:
                img = img[:, :, 0]

            self.total = s[0] * s[1]
            #img = img.reshape((self.total,))
            #img = "".join(map(str,img.tolist()))
            #img = img.tolist()
            huff_dict, total_bits = self.huffman_coding(img)
            self.infoE.setText("Huffman = " + str(np.round(total_bits/self.total,4)) +" bpp" )


if __name__ == "__main__":
    array = np.linspace(-1, 1, 256 * 256*3)

    # reshape to 2d
    mat = np.reshape(array, (256, 256, 3))

    # Creates PIL image
    img = np.uint8(mat * 150)
    WidgetPreview(Huffman).run(img.astype('double'))
