import numpy
import cv2
import scipy.fftpack as sfft
import Orange.data
from Orange.widgets.widget import OWWidget, Input, Output
from Orange.widgets.utils.widgetpreview import WidgetPreview
from Orange.widgets import gui, settings

class IDCT(OWWidget):
    name = "IDCT"
    description = "Calculate the Inverse Discrete Cosine Transform (II) of a image."
    icon = "icons/IDCT.svg"
    priority = 20

    class Inputs:
        a = Input("A", numpy.ndarray)

    class Outputs:
        idctA = Output("var(A)", numpy.ndarray)

    want_main_area = False
    proportion = settings.Setting(50)
    mode = settings.Setting(0)
    # [end-snippet-1]

    def __init__(self):
        super().__init__()

        self.dataset = None
        self.sample = None

        # GUI
        # [start-snippet-3]
        box = gui.widgetBox(self.controlArea, "Info")
        self.infoa = gui.widgetLabel(box, "No data on input yet, waiting to get something.")


    @Inputs.a
    def set_data_a(self, a):
        if a is not None:
            self.infoa.setText("input A is correct")
            self.a = a
        else:
            self.infoa.setText("No data on input yet, waiting to get something.")

    def DCTOpencv(self,img,M,N,inverse):
            inp = img.astype("double")/255
            flags = cv2.DCT_INVERSE if inverse else 0
            (m,n) = inp.shape[:2]
            out = numpy.zeros((m,n))
            for i in range(0,m,M):
                for j in range(0,n,N):
                    out[i:i+M,j:j+N] = cv2.dct(inp[i:i+M,j:j+N],flags =flags)
            out = out*255
            ImgOut = out
            '''ImgOut = numpy.zeros((m,n,3))
            ImgOut[:, :, 0] = out
            ImgOut[:, :, 1] = out
            ImgOut[:, :, 2] = out'''
            return ImgOut
    


    def handleNewSignals(self):
        if (self.a is not None):
            newidct = self.DCTOpencv(self.a,8,8,True)
            #self.Outputs.idctA.send(cv2.cvtColor(newidct.astype("uint8"), cv2.COLOR_GRAY2BGR))
            self.Outputs.idctA.send(newidct)

    # def commit(self):
    #     idct_value, type = idct(self.a)
    #     self.Outputs.sum.send(idct_value)
    #
    # def checkCommit(self):
    #     if self.commitOnChange:
    #         self.commit()

if __name__ == "__main__":
    WidgetPreview(Multiplier).run(set_data_b = 5,set_data_a=5)

