import math

import numpy
import cv2 as cv
import Orange.data
from Orange.widgets.widget import OWWidget, Input, Output
from Orange.widgets.utils.widgetpreview import WidgetPreview
from Orange.widgets import gui, settings
weights = numpy.array(
         [[8, 16, 19, 22, 26, 27, 29, 34], [16, 16, 22, 24, 27, 29, 34, 37], [19, 22, 26, 27, 29, 34, 34, 38],[22, 22, 26, 27, 29, 34, 37, 40], [22, 26, 27, 29, 32, 35, 40, 48], [26, 27, 29, 32, 35, 40, 48, 58],[26, 27, 29, 34, 38, 46, 56, 69], [27, 29, 35, 38, 46, 56, 69, 83]])

class Quant(OWWidget):
    name = "Quantizer"
    description = "Works as a quantizer."
    icon = "icons/Quantizer.svg"
    priority = 20


    class Inputs:
        a = Input("A", numpy.ndarray)

    class Outputs:
        qA = Output("Q(A)", numpy.ndarray)

    want_main_area = False
    ptype = settings.Setting(0)
    Sq = settings.Setting(2)
    SqVals = range(2, 64, 2)
    MODES = ['Inter', 'Intra']

    def __init__(self):
        super().__init__()

        self.dataset = None
        self.sample = None

        box = gui.widgetBox(self.controlArea, "Info")
        self.infoa = gui.widgetLabel(box, "No data on input yet, waiting to get something.")
        gui.separator(self.controlArea)

        gui.comboBox(self, self, 'ptype', items=self.MODES, callback=self.handleNewSignals)

        gui.comboBox(self, self, 'Sq', items=self.SqVals, callback=self.handleNewSignals)

    @Inputs.a
    def set_data_a(self, a):
        if a is not None:
            self.infoa.setText("input A is correct")
            self.a = a

    # def quant(self, image, predtype):
    #     weights = numpy.ones((8, 8))
    #     block = numpy.zeros((8, 8))
    #     blockq = numpy.zeros((8, 8))
    #     imares = image.copy().astype("double")
    #     Sq = 16
    #     if predtype:
    #         weights = 16 * weights
    #     else:
    #         weights = numpy.array(
    #             [[8, 16, 19, 22, 26, 27, 29, 34], [16, 16, 22, 24, 27, 29, 34, 37], [19, 22, 26, 27, 29, 34, 34, 38],
    #              [22, 22, 26, 27, 29, 34, 37, 40], [22, 26, 27, 29, 32, 35, 40, 48], [26, 27, 29, 32, 35, 40, 48, 58],
    #              [26, 27, 29, 34, 38, 46, 56, 69], [27, 29, 35, 38, 46, 56, 69, 83]])
    #     dim = image.shape
    #     filas = dim[0]
    #     columnas = dim[1]
    #     for ypos in range(0, filas, 8):
    #         f_ini = ypos
    #         f_fin = min(ypos + 8, filas)
    #         for xpos in range(0, columnas, 8):
    #             c_ini = xpos
    #             c_fin = min(xpos + 8, columnas)
    #             # Ya tenemos los bloques
    #             block = image[f_ini:f_fin, c_ini:c_fin]
    #             shapeb = numpy.shape(block)
    #             blockq = (8 * block)/(weights[0:shapeb[0],0:shapeb[1]] * Sq)
    #             block = image[f_ini:f_fin, c_ini:c_fin].copy().astype("double")
    #             blockq = (8 * block)/(weights * Sq)
    #             valueDC = blockq[0, 0]
    #             if predtype:
    #                 blockq = numpy.around(((2 * blockq + numpy.sign(blockq)) * weights * Sq) / 16)
    #             else:
    #                 blockq = numpy.around((blockq * weights * Sq) / 8)
    #                 blockq[0, 0] = valueDC * 8
    #             imares[f_ini:f_fin, c_ini:c_fin] = blockq
    #     return cv.cvtColor(imares, cv.COLOR_GRAY2BGR)

    def quanth261(self):
        th = 8
        q = self.q
        img = self.a.copy()
        block = numpy.zeros((8, 8))
        blockq = numpy.zeros((8, 8))
        imares = img.copy()
        dim = img.shape
        filas = dim[0]
        columnas = dim[1]
        print("The q value is: ", q)
        for ypos in range(0, filas, 8):
            f_ini = ypos
            f_fin = min(ypos + 8, filas)
            for xpos in range(0, columnas, 8):
                c_ini = xpos
                c_fin = min(xpos + 8, columnas)
                block = img[f_ini:f_fin, c_ini:c_fin]
                blockq = block
                #if (self.MODES[self.ptype] == "Inter"):
                #    blockqt = numpy.where(numpy.abs(block)<=th, 0, blockq)
                #     blockq = numpy.sign(blockqt)*numpy.ceil((numpy.abs(blockqt)-th)/q)
                #     blockq = numpy.round(blockq)
                if (self.MODES[self.ptype] == "Inter"):
                    blockqt=numpy.trunc(8*blockq/(16*q))

                if (self.MODES[self.ptype] == "Intra"):
                    blockqt=8*blockq/(weights*q)
                    blockqt[0,0] = blockq[0,0]/8
                    blockqt =numpy.sign(blockqt)*numpy.ceil(numpy.abs(blockqt)-0.5) #Rounding half toward zero
                # if (self.MODES[self.ptype] == "Intra"):
                #     blockqt = (8*blockq)/(weights*q)
                #     blockqt[0,0] = blockq[0,0]/8
                #     blockq = numpy.round(blockqt)
                imares[f_ini:f_fin, c_ini:c_fin] = blockqt
        return imares

    def handleNewSignals(self):
        if (self.a is not None):
            self.q = self.SqVals[self.Sq]
            newq = self.quanth261()
            nullcoeficients= numpy.sum(numpy.round(newq) == 0)
            self.infoa.setText("Null coeficients"+str(nullcoeficients))
            self.Outputs.qA.send(newq.astype('double'))

