import numpy as np
import math
import cv2 as cv
import Orange.data
from Orange.widgets.widget import OWWidget, Input, Output
from Orange.widgets.utils.widgetpreview import WidgetPreview
from Orange.widgets import gui, settings

class Stats(OWWidget):
    name = "Stats"
    description = "Shows Mean, Variance, RMS and MSE of a given image"
    icon = "icons/statistics.svg"
    priority = 20

    class Inputs:
        a = Input("Image (estimated)", np.ndarray)

    want_main_area = False
    def __init__(self):
        super().__init__()

        self.dataset = None
        self.sample = None

        box = gui.widgetBox(self.controlArea, "Stats:")
        self.infoM = gui.widgetLabel(box, "Mean = ")
        self.infoMax = gui.widgetLabel(box, "Max = ")
        self.infoMin = gui.widgetLabel(box, "Min = ")
        self.infoV = gui.widgetLabel(box, "Variance = ")
        self.infoMSE = gui.widgetLabel(box, "MSE = ")
        self.infoRMS = gui.widgetLabel(box, "RMS = ")
        self.infoShape = gui.widgetLabel(box, "Shape = ")

    @Inputs.a
    def set_data_a(self, a):
        if a is not None:
            self.a = a
            self.operate()

    def caster(self, inimag, type):
        res = inimag.astype(type)
        return res

    def operate(self):
        imaa = self.a.copy()
        #if (imaa.ndim > 1):
        #   imaa = cv.cvtColor(self.a, cv.COLOR_BGR2GRAY)
        imaa = self.caster(self.a, 'double')
        meana = np.mean(imaa)
        self.infoM.setText("Mean = " + str(meana))
        var = np.round(np.mean(np.subtract(imaa, meana) ** 2), 4)
        self.infoV.setText("Variance = " + str(var))
        mse = np.round(np.mean(imaa** 2), 4)
        self.infoMSE.setText("MSE = " + str(mse))
        rms = np.round(np.sqrt(mse), 4)
        self.infoRMS.setText("RMS = " + str(rms))
        Max = np.round(np.max(self.a), 4)
        self.infoMax.setText("Max = " + str(Max))
        Min = np.round(np.min(self.a), 4)
        self.infoMin.setText("Min = " + str(Min))
        shape = np.shape(self.a)
        self.infoShape.setText("Shape = " + str(shape))

    def handleNewSignals(self):
        if self.Inputs.a is not None:
            self.operate()
        else:
            self.infoM.setText("Mean = ")
            self.infoV.setText("Variance = ")
            self.infoMSE.setText("MSE = ")
            self.infoRMS.setText("RMS = ")
            self.infoMax.setText("Max = ")
            self.infoMin.setText("Min = ")
            self.infoShape.setText("Shape = ")


