import numpy
import cv2 as cv
import Orange.data
from orangewidget.widget import OWBaseWidget, Input, Output
from orangewidget.utils.widgetpreview import WidgetPreview
from orangewidget import gui, settings
from AnyQt import QtCore
from AnyQt import QtWidgets, QtGui

class YCrCb(OWBaseWidget):
    name = "YCrCb"
    description = "This module extracts the channel (YCbCr) of a given image"
    icon = "icons/YCrCb.svg"
    priority = 10
    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    #   with a fixed non resizable geometry.
    resizing_enabled = False
    Luminance = settings.Setting(0)
    Cr = settings.Setting(0)
    Cb = settings.Setting(0)

    class Inputs:
        imageIn = Input("Image", numpy.ndarray)

    class Outputs:
        imageOut = Output("Set image", numpy.ndarray)

    def __init__(self):
        super().__init__()

        # GUI
        self.Inputs.imageIn = None

        box = gui.widgetBox(self.controlArea, "YCrCb")
        self.ImageCanvas = gui.widgetLabel(box)

        gui.checkBox(
            self.controlArea, self, "Luminance", "Select luminance", callback=self.handleNewSignals
        )
        gui.checkBox(
            self.controlArea, self, "Cr", "Select Cr", callback=self.handleNewSignals
        )
        gui.checkBox(
            self.controlArea, self, "Cb", "Select Cb", callback=self.handleNewSignals
        )
        self.controlArea.setDisabled(True)


    @Inputs.imageIn
    def set_image(self, image):
        if image is None:
            self.controlArea.setDisabled(True)
            return
        else:
            self.controlArea.setDisabled(False)
            self.change_image(image)
            


    def change_image(self,image):
        image2 = image
        if (self.Luminance == True and ((self.Cr == False) and (self.Cb == False))):
            image2 = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
            #image2 = cv.cvtColor(image2, cv.COLOR_GRAY2BGR)
        else:
            image2 = cv.cvtColor(image2, cv.COLOR_BGR2YCrCb)
            if(self.Luminance == False):
                image2[:,:,0]=0
            if(self.Cr == False):
                image2[:,:,1]=0
            if(self.Cb == False):
                image2[:,:,2]=0
            #image2 = cv.cvtColor(image2, cv.COLOR_YCrCb2RGB)
        
        
        # size = image2.shape
        # step = image2.size / size[0]
        # qformat = QtGui.QImage.Format_Indexed8
        # if len(size) == 3:
        #     if size[2] == 4:
        #         qformat = QtGui.QImage.Format_RGBA8888
        #     else:
        #         qformat = QtGui.QImage.Format_RGB888
        #
        # img = QtGui.QImage(image2, size[1], size[0], step, qformat)
        # img = img.rgbSwapped()
        img = image2
        self.Inputs.imageIn=image
        self.Outputs.imageOut.send(img)
        # self.ImageCanvas.setPixmap(QtGui.QPixmap.fromImage(img))
        # self.resize(self.ImageCanvas.pixmap().size())


    def handleNewSignals(self):
        if (self.Inputs.imageIn is not None):
            self.set_image(self.Inputs.imageIn)
